//shading:         phong
//lighting model:  blinn
//lighting type:   directional

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)

#include "PhongDirectional.fxh"

//texture
texture Tex <string uiname="Texture fur";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

 texture Tex2 <string uiname="Texture color fur";>;
sampler Samp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex2);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float FurLength = 0;
float Layer     = 0; // 0 to 1 for the level
float scale;

float offset;

float3 vGravity = float3(0,-2.0,0);
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP: POSITION;
    float4 TexCd : TEXCOORD0;
    float3 LightDirV: TEXCOORD1;
    float3 NormV: TEXCOORD2;
    float3 ViewDirV: TEXCOORD3;
    float4 TexCd2 : TEXCOORD4;
    float4 TexCd3 : TEXCOORD5;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    float3 P = PosO.xyz + (NormO * FurLength);
    float3 normal =  normalize(mul(NormO, tW));
    vGravity = mul(vGravity, tW);
    float k =  pow(Layer, 3);
    P = P + vGravity*k;
    float4 znormal = 1 - dot(normal, float4(0,0,1,0));
    //inverse light direction in view space
    Out.PosWVP = mul(float4(P, 1.0f), tWVP);





    Out.LightDirV = normalize(-mul(lDir, tV));
    
    //normal in view space
    Out.NormV = normalize(mul(NormO, tWV));

    //position (projected)

    Out.TexCd =  TexCd * scale;
    Out.TexCd2 =  TexCd * scale + znormal * offset;
      Out.TexCd3 = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

vs2ps VS2(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    float3 P = PosO.xyz + (NormO * FurLength);
    float3 normal =  normalize(mul(NormO, tW));
    vGravity = mul(vGravity, tW);
    float k =  pow(Layer, 3);
    P = P + vGravity*k;
    float4 znormal = 1 - dot(normal, float4(0,0,1,0));
    //inverse light direction in view space
    Out.PosWVP = mul(float4(P, 1.0f), tWVP);





    Out.LightDirV = normalize(-mul(lDir, tV));

    //normal in view space
    Out.NormV = normalize(mul(NormO, tWV));

    //position (projected)

    Out.TexCd =  PosO * scale;
    Out.TexCd2 =  PosO * scale + znormal * offset;
    Out.TexCd3 = mul(PosO, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}
// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    float4 col = tex2D(Samp, In.TexCd);
     float4 colf = tex2D(Samp2, In.TexCd3);
    float4 colffset = tex2D(Samp, In.TexCd2);
    float4 colo = {col.rgb * col.a, 0};
    float4 colo2 = {1- colffset.rgb * colffset.a, 0};
    float cola =  col.a +  colffset.a;
    float4 fcolor = {colo.rgb + colo2.rgb,cola} ;
    fcolor.rgb *= colf;
     fcolor.rgb *= PhongDirectional(In.NormV, In.ViewDirV, In.LightDirV);


    return mul(fcolor, tColor);
}


// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TPhongDirectional_textcoord_from_file
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PS();
    }
}
technique TPhongDirectional_textcoord_from_Pos
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS2();
        PixelShader = compile ps_2_0 PS();
    }
}
